Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.ObjectModel
Imports System.Data.Services.Client
Imports System.Diagnostics
Imports System.Linq
Imports System.Windows
Imports System.Windows.Controls
Imports SilverlightAstoriaQA.NWServiceReference

Namespace SilverlightAstoriaQA
	Partial Public Class Page
		Inherits UserControl
		Private _products As ObservableCollection(Of Product) = New ObservableCollection(Of Product)()
		Private _ctx As New Entities(New Uri("NWDataService.svc", UriKind.Relative))
		Private _orderDetails As ObservableCollection(Of OrderDetail) = New ObservableCollection(Of OrderDetail)()

		Public Sub New()
			InitializeComponent()
			AddHandler Loaded, AddressOf Page_Loaded
			AddHandler loadButton.Click, AddressOf LoadButton_Clicked
			AddHandler saveButton.Click, AddressOf SaveButton_Clicked
			AddHandler productDataGrid.SelectionChanged, AddressOf productDataGrid_SelectionChanged
		End Sub

		Private Sub productDataGrid_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
			Dim product As Product = TryCast(productDataGrid.SelectedItem, Product)
			If product Is Nothing Then
				Return
			End If

			_orderDetails.Clear()

			If product.OrderDetails Is Nothing OrElse product.OrderDetails.Count = 0 Then
				_ctx.BeginLoadProperty(product, "OrderDetails", AddressOf FindOrderDetail_Completed, Nothing)
			Else
				LoadOrderDetails()
			End If
		End Sub

		Private Sub LoadOrderDetails()
			Dim product As Product = TryCast(productDataGrid.SelectedItem, Product)
			If product Is Nothing Then
				Return
			End If

			Dim query = ( _
					From od In product.OrderDetails _
					Order By od.OrderID Ascending _
					Select od)
			For Each item As OrderDetail In query
                AddHandler item.PropertyChanged, (AddressOf orderDetail_PropertyChanged)
				_orderDetails.Add(item)
			Next item
        End Sub

        Private Sub orderDetail_PropertyChanged(ByVal sender As Object, ByVal e As System.ComponentModel.PropertyChangedEventArgs)
            Dim entity As OrderDetail = CType(sender, OrderDetail)
            _ctx.UpdateObject(entity)
        End Sub

        Private Sub FindOrderDetail_Completed(ByVal result As IAsyncResult)
            _ctx.EndLoadProperty(result)
            Deployment.Current.Dispatcher.BeginInvoke(New Action(AddressOf LoadOrderDetails))
        End Sub

        Private Sub SaveButton_Clicked(ByVal sender As Object, ByVal e As RoutedEventArgs)
            _ctx.BeginSaveChanges(SaveChangesOptions.Batch, New AsyncCallback(AddressOf Save_Complete), Nothing)
        End Sub

        Private Sub Save_Complete(ByVal result As IAsyncResult)
            Dim succeeded As Boolean = True
            Try
                Dim response As DataServiceResponse = CType(_ctx.EndSaveChanges(result), DataServiceResponse)

                For Each opResponse As OperationResponse In response
                    If opResponse.Error IsNot Nothing Then
                        Debug.WriteLine(String.Concat("Failed to save: ", opResponse.StatusCode))
                        succeeded = False
                    End If
                Next opResponse
            Catch ex As Exception
                Debug.WriteLine(String.Concat("Save Operation Threw an Exception: ", ex))
                succeeded = False
            End Try

            If succeeded Then
                MessageBox.Show("Saved new socks!")
            Else
                MessageBox.Show("Failed to save new socks!")
            End If
        End Sub

        Private Sub LoadButton_Clicked(ByVal sender As Object, ByVal e As RoutedEventArgs)
            _ctx.MergeOption = MergeOption.OverwriteChanges ' makes sure we get fresh data and overwrite any changes (cancels local changes).
            FindProducts()
        End Sub

        Private Sub Page_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
            productDataGrid.DataContext = _products
            orderDetailsDataGrid.DataContext = _orderDetails
        End Sub

        Private Sub FindProducts()
            _products.Clear()
            Dim dq As DataServiceQuery(Of Product) = TryCast(( _
               From p In _ctx.ProductSet _
               Select p), DataServiceQuery(Of Product))
            dq.BeginExecute(New AsyncCallback(AddressOf FindProduct_Completed), dq)


            Return
            Dim dq1 As DataServiceQuery(Of Product) = TryCast(( _
               From p In _ctx.ProductSet _
               Select p), DataServiceQuery(Of Product))


            Dim dq3 As DataServiceQuery(Of Product) = TryCast(( _
               From p In _ctx.ProductSet.Expand("OrderDetails/Orders") _
               Select p), DataServiceQuery(Of Product))


        End Sub

        Private Sub FindProduct_Completed(ByVal result As IAsyncResult)
            Dim query As DataServiceQuery(Of Product) = CType(result.AsyncState, DataServiceQuery(Of Product))
            Try
                Dim entities = query.EndExecute(result)
                For Each item As Product In entities
                    AddHandler item.PropertyChanged, (AddressOf entity_PropertyChanged)
                    _products.Add(item)
                Next item
            Catch ex As Exception
                Debug.WriteLine("Failed to retrieve data: " & ex.ToString())
            End Try
        End Sub

        Private Sub entity_PropertyChanged(ByVal sender As Object, ByVal e As System.ComponentModel.PropertyChangedEventArgs)
            Dim entity As Product = CType(sender, Product)
            _ctx.UpdateObject(entity)
        End Sub

    End Class
End Namespace
